"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .inputaudio import InputAudio, InputAudioTypedDict
from .inputfile import InputFile, InputFileTypedDict
from .inputtext import InputText, InputTextTypedDict
from .inputvideo import InputVideo, InputVideoTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import get_discriminator, validate_open_enum
from pydantic import Discriminator, Tag, model_serializer
from pydantic.functional_validators import PlainValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


InputMessageItemTypeMessage = Literal["message",]


InputMessageItemRoleDeveloper = Literal["developer",]


InputMessageItemRoleSystem = Literal["system",]


InputMessageItemRoleUser = Literal["user",]


InputMessageItemRoleUnionTypedDict = TypeAliasType(
    "InputMessageItemRoleUnionTypedDict",
    Union[
        InputMessageItemRoleUser,
        InputMessageItemRoleSystem,
        InputMessageItemRoleDeveloper,
    ],
)


InputMessageItemRoleUnion = TypeAliasType(
    "InputMessageItemRoleUnion",
    Union[
        InputMessageItemRoleUser,
        InputMessageItemRoleSystem,
        InputMessageItemRoleDeveloper,
    ],
)


InputMessageItemContentType = Literal["input_image",]


InputMessageItemDetail = Union[
    Literal[
        "auto",
        "high",
        "low",
    ],
    UnrecognizedStr,
]


class InputMessageItemContentInputImageTypedDict(TypedDict):
    r"""Image input content item"""

    type: InputMessageItemContentType
    detail: InputMessageItemDetail
    image_url: NotRequired[Nullable[str]]


class InputMessageItemContentInputImage(BaseModel):
    r"""Image input content item"""

    type: InputMessageItemContentType

    detail: Annotated[InputMessageItemDetail, PlainValidator(validate_open_enum(False))]

    image_url: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["image_url"]
        nullable_fields = ["image_url"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


InputMessageItemContentUnionTypedDict = TypeAliasType(
    "InputMessageItemContentUnionTypedDict",
    Union[
        InputTextTypedDict,
        InputAudioTypedDict,
        InputVideoTypedDict,
        InputMessageItemContentInputImageTypedDict,
        InputFileTypedDict,
    ],
)


InputMessageItemContentUnion = Annotated[
    Union[
        Annotated[InputText, Tag("input_text")],
        Annotated[InputMessageItemContentInputImage, Tag("input_image")],
        Annotated[InputFile, Tag("input_file")],
        Annotated[InputAudio, Tag("input_audio")],
        Annotated[InputVideo, Tag("input_video")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class InputMessageItemTypedDict(TypedDict):
    role: InputMessageItemRoleUnionTypedDict
    id: NotRequired[str]
    type: NotRequired[InputMessageItemTypeMessage]
    content: NotRequired[Nullable[List[InputMessageItemContentUnionTypedDict]]]


class InputMessageItem(BaseModel):
    role: InputMessageItemRoleUnion

    id: Optional[str] = None

    type: Optional[InputMessageItemTypeMessage] = None

    content: OptionalNullable[List[InputMessageItemContentUnion]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "type", "content"]
        nullable_fields = ["content"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatcontenttext import ChatContentText, ChatContentTextTypedDict
from openrouter.types import BaseModel
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ChatDeveloperMessageRole = Literal["developer",]


ChatDeveloperMessageContentTypedDict = TypeAliasType(
    "ChatDeveloperMessageContentTypedDict", Union[str, List[ChatContentTextTypedDict]]
)
r"""Developer message content"""


ChatDeveloperMessageContent = TypeAliasType(
    "ChatDeveloperMessageContent", Union[str, List[ChatContentText]]
)
r"""Developer message content"""


class ChatDeveloperMessageTypedDict(TypedDict):
    r"""Developer message"""

    role: ChatDeveloperMessageRole
    content: ChatDeveloperMessageContentTypedDict
    r"""Developer message content"""
    name: NotRequired[str]
    r"""Optional name for the developer message"""


class ChatDeveloperMessage(BaseModel):
    r"""Developer message"""

    role: ChatDeveloperMessageRole

    content: ChatDeveloperMessageContent
    r"""Developer message content"""

    name: Optional[str] = None
    r"""Optional name for the developer message"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ChatFormatTextConfigType = Literal["text",]


class ChatFormatTextConfigTypedDict(TypedDict):
    r"""Default text response format"""

    type: ChatFormatTextConfigType


class ChatFormatTextConfig(BaseModel):
    r"""Default text response format"""

    type: ChatFormatTextConfigType



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .easyinputmessage import EasyInputMessage, EasyInputMessageTypedDict
from .functioncallitem import FunctionCallItem, FunctionCallItemTypedDict
from .functioncalloutputitem import (
    FunctionCallOutputItem,
    FunctionCallOutputItemTypedDict,
)
from .inputmessageitem import InputMessageItem, InputMessageItemTypedDict
from .openairesponsesrefusalcontent import (
    OpenAIResponsesRefusalContent,
    OpenAIResponsesRefusalContentTypedDict,
)
from .outputdatetimeitem import OutputDatetimeItem, OutputDatetimeItemTypedDict
from .outputfilesearchcallitem import (
    OutputFileSearchCallItem,
    OutputFileSearchCallItemTypedDict,
)
from .outputfunctioncallitem import (
    OutputFunctionCallItem,
    OutputFunctionCallItemTypedDict,
)
from .outputimagegenerationcallitem import (
    OutputImageGenerationCallItem,
    OutputImageGenerationCallItemTypedDict,
)
from .outputservertoolitem import OutputServerToolItem, OutputServerToolItemTypedDict
from .outputwebsearchcallitem import (
    OutputWebSearchCallItem,
    OutputWebSearchCallItemTypedDict,
)
from .reasoningitem import ReasoningItem, ReasoningItemTypedDict
from .reasoningsummarytext import ReasoningSummaryText, ReasoningSummaryTextTypedDict
from .reasoningtextcontent import ReasoningTextContent, ReasoningTextContentTypedDict
from .responseoutputtext import ResponseOutputText, ResponseOutputTextTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import get_discriminator, validate_open_enum
import pydantic
from pydantic import Discriminator, Tag, model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


InputsTypeReasoning = Literal["reasoning",]


InputsStatusInProgress2 = Literal["in_progress",]


InputsStatusIncomplete2 = Literal["incomplete",]


InputsStatusCompleted2 = Literal["completed",]


InputsStatusUnion2TypedDict = TypeAliasType(
    "InputsStatusUnion2TypedDict",
    Union[InputsStatusCompleted2, InputsStatusIncomplete2, InputsStatusInProgress2],
)


InputsStatusUnion2 = TypeAliasType(
    "InputsStatusUnion2",
    Union[InputsStatusCompleted2, InputsStatusIncomplete2, InputsStatusInProgress2],
)


InputsFormat = Union[
    Literal[
        "unknown",
        "openai-responses-v1",
        "azure-openai-responses-v1",
        "xai-responses-v1",
        "anthropic-claude-v1",
        "google-gemini-v1",
    ],
    UnrecognizedStr,
]
r"""The format of the reasoning content"""


class InputsReasoningTypedDict(TypedDict):
    r"""An output item containing reasoning"""

    type: InputsTypeReasoning
    id: str
    summary: Nullable[List[ReasoningSummaryTextTypedDict]]
    content: NotRequired[Nullable[List[ReasoningTextContentTypedDict]]]
    encrypted_content: NotRequired[Nullable[str]]
    status: NotRequired[InputsStatusUnion2TypedDict]
    signature: NotRequired[Nullable[str]]
    r"""A signature for the reasoning content, used for verification"""
    format_: NotRequired[Nullable[InputsFormat]]
    r"""The format of the reasoning content"""


class InputsReasoning(BaseModel):
    r"""An output item containing reasoning"""

    type: InputsTypeReasoning

    id: str

    summary: Nullable[List[ReasoningSummaryText]]

    content: OptionalNullable[List[ReasoningTextContent]] = UNSET

    encrypted_content: OptionalNullable[str] = UNSET

    status: Optional[InputsStatusUnion2] = None

    signature: OptionalNullable[str] = UNSET
    r"""A signature for the reasoning content, used for verification"""

    format_: Annotated[
        Annotated[
            OptionalNullable[InputsFormat], PlainValidator(validate_open_enum(False))
        ],
        pydantic.Field(alias="format"),
    ] = UNSET
    r"""The format of the reasoning content"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "content",
            "encrypted_content",
            "status",
            "signature",
            "format",
        ]
        nullable_fields = [
            "content",
            "summary",
            "encrypted_content",
            "signature",
            "format",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


InputsRole = Literal["assistant",]


InputsTypeMessage = Literal["message",]


InputsStatusInProgress1 = Literal["in_progress",]


InputsStatusIncomplete1 = Literal["incomplete",]


InputsStatusCompleted1 = Literal["completed",]


InputsStatusUnion1TypedDict = TypeAliasType(
    "InputsStatusUnion1TypedDict",
    Union[InputsStatusCompleted1, InputsStatusIncomplete1, InputsStatusInProgress1],
)


InputsStatusUnion1 = TypeAliasType(
    "InputsStatusUnion1",
    Union[InputsStatusCompleted1, InputsStatusIncomplete1, InputsStatusInProgress1],
)


InputsContent1TypedDict = TypeAliasType(
    "InputsContent1TypedDict",
    Union[OpenAIResponsesRefusalContentTypedDict, ResponseOutputTextTypedDict],
)


InputsContent1 = Annotated[
    Union[
        Annotated[ResponseOutputText, Tag("output_text")],
        Annotated[OpenAIResponsesRefusalContent, Tag("refusal")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


InputsContent2TypedDict = TypeAliasType(
    "InputsContent2TypedDict", Union[List[InputsContent1TypedDict], str, Any]
)


InputsContent2 = TypeAliasType("InputsContent2", Union[List[InputsContent1], str, Any])


InputsPhaseFinalAnswer = Literal["final_answer",]


InputsPhaseCommentary = Literal["commentary",]


InputsPhaseUnionTypedDict = TypeAliasType(
    "InputsPhaseUnionTypedDict",
    Union[InputsPhaseCommentary, InputsPhaseFinalAnswer, Any],
)
r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


InputsPhaseUnion = TypeAliasType(
    "InputsPhaseUnion", Union[InputsPhaseCommentary, InputsPhaseFinalAnswer, Any]
)
r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


class InputsMessageTypedDict(TypedDict):
    r"""An output message item"""

    id: str
    role: InputsRole
    type: InputsTypeMessage
    content: Nullable[InputsContent2TypedDict]
    status: NotRequired[InputsStatusUnion1TypedDict]
    phase: NotRequired[Nullable[InputsPhaseUnionTypedDict]]
    r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


class InputsMessage(BaseModel):
    r"""An output message item"""

    id: str

    role: InputsRole

    type: InputsTypeMessage

    content: Nullable[InputsContent2]

    status: Optional[InputsStatusUnion1] = None

    phase: OptionalNullable[InputsPhaseUnion] = UNSET
    r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["status", "phase"]
        nullable_fields = ["content", "phase"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


InputsUnion1TypedDict = TypeAliasType(
    "InputsUnion1TypedDict",
    Union[
        OutputWebSearchCallItemTypedDict,
        EasyInputMessageTypedDict,
        InputMessageItemTypedDict,
        OutputServerToolItemTypedDict,
        OutputImageGenerationCallItemTypedDict,
        OutputFileSearchCallItemTypedDict,
        FunctionCallOutputItemTypedDict,
        OutputDatetimeItemTypedDict,
        FunctionCallItemTypedDict,
        OutputFunctionCallItemTypedDict,
        InputsMessageTypedDict,
        InputsReasoningTypedDict,
        ReasoningItemTypedDict,
    ],
)


InputsUnion1 = TypeAliasType(
    "InputsUnion1",
    Union[
        OutputWebSearchCallItem,
        EasyInputMessage,
        InputMessageItem,
        OutputServerToolItem,
        OutputImageGenerationCallItem,
        OutputFileSearchCallItem,
        FunctionCallOutputItem,
        OutputDatetimeItem,
        FunctionCallItem,
        OutputFunctionCallItem,
        InputsMessage,
        InputsReasoning,
        ReasoningItem,
    ],
)


InputsUnionTypedDict = TypeAliasType(
    "InputsUnionTypedDict", Union[str, List[InputsUnion1TypedDict]]
)
r"""Input for a response request - can be a string or array of items"""


InputsUnion = TypeAliasType("InputsUnion", Union[str, List[InputsUnion1]])
r"""Input for a response request - can be a string or array of items"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing import Literal, Union
from typing_extensions import Annotated, TypedDict


ComputerUseServerToolType = Literal["computer_use_preview",]


Environment = Union[
    Literal[
        "windows",
        "mac",
        "linux",
        "ubuntu",
        "browser",
    ],
    UnrecognizedStr,
]


class ComputerUseServerToolTypedDict(TypedDict):
    r"""Computer use preview tool configuration"""

    type: ComputerUseServerToolType
    display_height: float
    display_width: float
    environment: Environment


class ComputerUseServerTool(BaseModel):
    r"""Computer use preview tool configuration"""

    type: ComputerUseServerToolType

    display_height: float

    display_width: float

    environment: Annotated[Environment, PlainValidator(validate_open_enum(False))]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ResponseFormatTextGrammarType = Literal["grammar",]


class ResponseFormatTextGrammarTypedDict(TypedDict):
    r"""Custom grammar response format"""

    type: ResponseFormatTextGrammarType
    grammar: str
    r"""Custom grammar for text generation"""


class ResponseFormatTextGrammar(BaseModel):
    r"""Custom grammar response format"""

    type: ResponseFormatTextGrammarType

    grammar: str
    r"""Custom grammar for text generation"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.components import (
    openresponsesresult as components_openresponsesresult,
    responsesrequest as components_responsesrequest,
    streamevents as components_streamevents,
)
from openrouter.types import BaseModel
from openrouter.utils import (
    FieldMetadata,
    HeaderMetadata,
    RequestMetadata,
    eventstreaming,
)
import pydantic
from typing import Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class CreateResponsesGlobalsTypedDict(TypedDict):
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class CreateResponsesGlobals(BaseModel):
    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class CreateResponsesRequestTypedDict(TypedDict):
    responses_request: components_responsesrequest.ResponsesRequestTypedDict
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class CreateResponsesRequest(BaseModel):
    responses_request: Annotated[
        components_responsesrequest.ResponsesRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class CreateResponsesResponseBodyTypedDict(TypedDict):
    r"""Successful response"""

    data: components_streamevents.StreamEventsTypedDict
    r"""Union of all possible event types emitted during response streaming"""


class CreateResponsesResponseBody(BaseModel):
    r"""Successful response"""

    data: components_streamevents.StreamEvents
    r"""Union of all possible event types emitted during response streaming"""


CreateResponsesResponseTypedDict = TypeAliasType(
    "CreateResponsesResponseTypedDict",
    Union[
        components_openresponsesresult.OpenResponsesResultTypedDict,
        Union[
            eventstreaming.EventStream[components_streamevents.StreamEventsTypedDict],
            eventstreaming.EventStreamAsync[
                components_streamevents.StreamEventsTypedDict
            ],
        ],
    ],
)


CreateResponsesResponse = TypeAliasType(
    "CreateResponsesResponse",
    Union[
        components_openresponsesresult.OpenResponsesResult,
        Union[
            eventstreaming.EventStream[components_streamevents.StreamEvents],
            eventstreaming.EventStreamAsync[components_streamevents.StreamEvents],
        ],
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatmessagecontentitemaudio import (
    ChatMessageContentItemAudio,
    ChatMessageContentItemAudioTypedDict,
)
from .chatmessagecontentitemfile import (
    ChatMessageContentItemFile,
    ChatMessageContentItemFileTypedDict,
)
from .chatmessagecontentitemimage import (
    ChatMessageContentItemImage,
    ChatMessageContentItemImageTypedDict,
)
from .chatmessagecontentitemtext import (
    ChatMessageContentItemText,
    ChatMessageContentItemTextTypedDict,
)
from .chatmessagecontentitemvideo import (
    ChatMessageContentItemVideo,
    ChatMessageContentItemVideoTypedDict,
)
from .chatmessagecontentitemvideolegacy import (
    ChatMessageContentItemVideoLegacy,
    ChatMessageContentItemVideoLegacyTypedDict,
)
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


ChatMessageContentItem1TypedDict = TypeAliasType(
    "ChatMessageContentItem1TypedDict",
    Union[
        ChatMessageContentItemVideoLegacyTypedDict, ChatMessageContentItemVideoTypedDict
    ],
)


ChatMessageContentItem1 = Annotated[
    Union[
        Annotated[ChatMessageContentItemVideoLegacy, Tag("input_video")],
        Annotated[ChatMessageContentItemVideo, Tag("video_url")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


ChatMessageContentItemTypedDict = TypeAliasType(
    "ChatMessageContentItemTypedDict",
    Union[
        ChatMessageContentItemImageTypedDict,
        ChatMessageContentItemAudioTypedDict,
        ChatMessageContentItemFileTypedDict,
        ChatMessageContentItemTextTypedDict,
        ChatMessageContentItem1TypedDict,
    ],
)
r"""Content part for chat completion messages"""


ChatMessageContentItem = TypeAliasType(
    "ChatMessageContentItem",
    Union[
        ChatMessageContentItemImage,
        ChatMessageContentItemAudio,
        ChatMessageContentItemFile,
        ChatMessageContentItemText,
        ChatMessageContentItem1,
    ],
)
r"""Content part for chat completion messages"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


OpenResponsesImageGenerationToolType = Literal["image_generation",]


Background = Union[
    Literal[
        "transparent",
        "opaque",
        "auto",
    ],
    UnrecognizedStr,
]


InputFidelity = Union[
    Literal[
        "high",
        "low",
    ],
    UnrecognizedStr,
]


class InputImageMaskTypedDict(TypedDict):
    image_url: NotRequired[str]
    file_id: NotRequired[str]


class InputImageMask(BaseModel):
    image_url: Optional[str] = None

    file_id: Optional[str] = None


ModelEnum = Union[
    Literal[
        "gpt-image-1",
        "gpt-image-1-mini",
    ],
    UnrecognizedStr,
]


Moderation = Union[
    Literal[
        "auto",
        "low",
    ],
    UnrecognizedStr,
]


OutputFormat = Union[
    Literal[
        "png",
        "webp",
        "jpeg",
    ],
    UnrecognizedStr,
]


Quality = Union[
    Literal[
        "low",
        "medium",
        "high",
        "auto",
    ],
    UnrecognizedStr,
]


Size = Union[
    Literal[
        "1024x1024",
        "1024x1536",
        "1536x1024",
        "auto",
    ],
    UnrecognizedStr,
]


class OpenResponsesImageGenerationToolTypedDict(TypedDict):
    r"""Image generation tool configuration"""

    type: OpenResponsesImageGenerationToolType
    background: NotRequired[Background]
    input_fidelity: NotRequired[Nullable[InputFidelity]]
    input_image_mask: NotRequired[InputImageMaskTypedDict]
    model: NotRequired[ModelEnum]
    moderation: NotRequired[Moderation]
    output_compression: NotRequired[float]
    output_format: NotRequired[OutputFormat]
    partial_images: NotRequired[float]
    quality: NotRequired[Quality]
    size: NotRequired[Size]


class OpenResponsesImageGenerationTool(BaseModel):
    r"""Image generation tool configuration"""

    type: OpenResponsesImageGenerationToolType

    background: Annotated[
        Optional[Background], PlainValidator(validate_open_enum(False))
    ] = None

    input_fidelity: Annotated[
        OptionalNullable[InputFidelity], PlainValidator(validate_open_enum(False))
    ] = UNSET

    input_image_mask: Optional[InputImageMask] = None

    model: Annotated[Optional[ModelEnum], PlainValidator(validate_open_enum(False))] = (
        None
    )

    moderation: Annotated[
        Optional[Moderation], PlainValidator(validate_open_enum(False))
    ] = None

    output_compression: Optional[float] = None

    output_format: Annotated[
        Optional[OutputFormat], PlainValidator(validate_open_enum(False))
    ] = None

    partial_images: Optional[float] = None

    quality: Annotated[Optional[Quality], PlainValidator(validate_open_enum(False))] = (
        None
    )

    size: Annotated[Optional[Size], PlainValidator(validate_open_enum(False))] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "background",
            "input_fidelity",
            "input_image_mask",
            "model",
            "moderation",
            "output_compression",
            "output_format",
            "partial_images",
            "quality",
            "size",
        ]
        nullable_fields = ["input_fidelity"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatmessagecontentitemtext import (
    ChatMessageContentItemText,
    ChatMessageContentItemTextTypedDict,
)
from openrouter.types import BaseModel
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


DeveloperMessageRole = Literal["developer",]


DeveloperMessageContentTypedDict = TypeAliasType(
    "DeveloperMessageContentTypedDict",
    Union[str, List[ChatMessageContentItemTextTypedDict]],
)
r"""Developer message content"""


DeveloperMessageContent = TypeAliasType(
    "DeveloperMessageContent", Union[str, List[ChatMessageContentItemText]]
)
r"""Developer message content"""


class DeveloperMessageTypedDict(TypedDict):
    r"""Developer message"""

    role: DeveloperMessageRole
    content: DeveloperMessageContentTypedDict
    r"""Developer message content"""
    name: NotRequired[str]
    r"""Optional name for the developer message"""


class DeveloperMessage(BaseModel):
    r"""Developer message"""

    role: DeveloperMessageRole

    content: DeveloperMessageContent
    r"""Developer message content"""

    name: Optional[str] = None
    r"""Optional name for the developer message"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


WebSearchCallSearchingEventType = Literal["response.web_search_call.searching",]


class WebSearchCallSearchingEventTypedDict(TypedDict):
    r"""Web search call is searching"""

    type: WebSearchCallSearchingEventType
    item_id: str
    output_index: float
    sequence_number: float


class WebSearchCallSearchingEvent(BaseModel):
    r"""Web search call is searching"""

    type: WebSearchCallSearchingEventType

    item_id: str

    output_index: float

    sequence_number: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import UnrecognizedStr
from openrouter.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing import Literal, Union
from typing_extensions import Annotated, TypeAliasType


PDFParserEnginePDFText = Literal["pdf-text",]


PDFParserEngineEnum = Union[
    Literal[
        "mistral-ocr",
        "native",
        "cloudflare-ai",
    ],
    UnrecognizedStr,
]


PDFParserEngineTypedDict = TypeAliasType(
    "PDFParserEngineTypedDict", Union[PDFParserEngineEnum, PDFParserEnginePDFText]
)
r"""The engine to use for parsing PDF files. \"pdf-text\" is deprecated and automatically redirected to \"cloudflare-ai\"."""


PDFParserEngine = TypeAliasType(
    "PDFParserEngine",
    Union[
        Annotated[PDFParserEngineEnum, PlainValidator(validate_open_enum(False))],
        PDFParserEnginePDFText,
    ],
)
r"""The engine to use for parsing PDF files. \"pdf-text\" is deprecated and automatically redirected to \"cloudflare-ai\"."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ReasoningSummaryTextDoneEventType = Literal["response.reasoning_summary_text.done",]


class ReasoningSummaryTextDoneEventTypedDict(TypedDict):
    r"""Event emitted when reasoning summary text streaming is complete"""

    type: ReasoningSummaryTextDoneEventType
    item_id: str
    output_index: float
    summary_index: float
    text: str
    sequence_number: float


class ReasoningSummaryTextDoneEvent(BaseModel):
    r"""Event emitted when reasoning summary text streaming is complete"""

    type: ReasoningSummaryTextDoneEventType

    item_id: str

    output_index: float

    summary_index: float

    text: str

    sequence_number: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatmessagecontentitem import (
    ChatMessageContentItem,
    ChatMessageContentItemTypedDict,
)
from openrouter.types import BaseModel
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


UserMessageRole = Literal["user",]


UserMessageContentTypedDict = TypeAliasType(
    "UserMessageContentTypedDict", Union[str, List[ChatMessageContentItemTypedDict]]
)
r"""User message content"""


UserMessageContent = TypeAliasType(
    "UserMessageContent", Union[str, List[ChatMessageContentItem]]
)
r"""User message content"""


class UserMessageTypedDict(TypedDict):
    r"""User message"""

    role: UserMessageRole
    content: UserMessageContentTypedDict
    r"""User message content"""
    name: NotRequired[str]
    r"""Optional name for the user"""


class UserMessage(BaseModel):
    r"""User message"""

    role: UserMessageRole

    content: UserMessageContent
    r"""User message content"""

    name: Optional[str] = None
    r"""Optional name for the user"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


OpenResponsesMcpToolType = Literal["mcp",]


class AllowedToolsTypedDict(TypedDict):
    tool_names: NotRequired[List[str]]
    read_only: NotRequired[bool]


class AllowedTools(BaseModel):
    tool_names: Optional[List[str]] = None

    read_only: Optional[bool] = None


ConnectorID = Union[
    Literal[
        "connector_dropbox",
        "connector_gmail",
        "connector_googlecalendar",
        "connector_googledrive",
        "connector_microsoftteams",
        "connector_outlookcalendar",
        "connector_outlookemail",
        "connector_sharepoint",
    ],
    UnrecognizedStr,
]


RequireApprovalNever = Literal["never",]


RequireApprovalAlways = Literal["always",]


class NeverTypedDict(TypedDict):
    tool_names: NotRequired[List[str]]


class Never(BaseModel):
    tool_names: Optional[List[str]] = None


class AlwaysTypedDict(TypedDict):
    tool_names: NotRequired[List[str]]


class Always(BaseModel):
    tool_names: Optional[List[str]] = None


class RequireApprovalTypedDict(TypedDict):
    never: NotRequired[NeverTypedDict]
    always: NotRequired[AlwaysTypedDict]


class RequireApproval(BaseModel):
    never: Optional[Never] = None

    always: Optional[Always] = None


class OpenResponsesMcpToolTypedDict(TypedDict):
    r"""MCP (Model Context Protocol) tool configuration"""

    type: OpenResponsesMcpToolType
    server_label: str
    allowed_tools: NotRequired[Nullable[Any]]
    authorization: NotRequired[str]
    connector_id: NotRequired[ConnectorID]
    headers: NotRequired[Nullable[Dict[str, str]]]
    require_approval: NotRequired[Nullable[Any]]
    server_description: NotRequired[str]
    server_url: NotRequired[str]


class OpenResponsesMcpTool(BaseModel):
    r"""MCP (Model Context Protocol) tool configuration"""

    type: OpenResponsesMcpToolType

    server_label: str

    allowed_tools: OptionalNullable[Any] = UNSET

    authorization: Optional[str] = None

    connector_id: Annotated[
        Optional[ConnectorID], PlainValidator(validate_open_enum(False))
    ] = None

    headers: OptionalNullable[Dict[str, str]] = UNSET

    require_approval: OptionalNullable[Any] = UNSET

    server_description: Optional[str] = None

    server_url: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "allowed_tools",
            "authorization",
            "connector_id",
            "headers",
            "require_approval",
            "server_description",
            "server_url",
        ]
        nullable_fields = ["allowed_tools", "headers", "require_approval"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pdfparserengine import PDFParserEngine, PDFParserEngineTypedDict
from openrouter.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PDFParserOptionsTypedDict(TypedDict):
    r"""Options for PDF parsing."""

    engine: NotRequired[PDFParserEngineTypedDict]
    r"""The engine to use for parsing PDF files. \"pdf-text\" is deprecated and automatically redirected to \"cloudflare-ai\"."""


class PDFParserOptions(BaseModel):
    r"""Options for PDF parsing."""

    engine: Optional[PDFParserEngine] = None
    r"""The engine to use for parsing PDF files. \"pdf-text\" is deprecated and automatically redirected to \"cloudflare-ai\"."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatmessagecontentitem import (
    ChatMessageContentItem,
    ChatMessageContentItemTypedDict,
)
from openrouter.types import BaseModel
from typing import List, Literal, Union
from typing_extensions import TypeAliasType, TypedDict


ToolResponseMessageRole = Literal["tool",]


ToolResponseMessageContentTypedDict = TypeAliasType(
    "ToolResponseMessageContentTypedDict",
    Union[str, List[ChatMessageContentItemTypedDict]],
)
r"""Tool response content"""


ToolResponseMessageContent = TypeAliasType(
    "ToolResponseMessageContent", Union[str, List[ChatMessageContentItem]]
)
r"""Tool response content"""


class ToolResponseMessageTypedDict(TypedDict):
    r"""Tool response message"""

    role: ToolResponseMessageRole
    content: ToolResponseMessageContentTypedDict
    r"""Tool response content"""
    tool_call_id: str
    r"""ID of the assistant message tool call this message responds to"""


class ToolResponseMessage(BaseModel):
    r"""Tool response message"""

    role: ToolResponseMessageRole

    content: ToolResponseMessageContent
    r"""Tool response content"""

    tool_call_id: str
    r"""ID of the assistant message tool call this message responds to"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


ChatStreamingMessageToolCallType = Literal["function",]
r"""Tool call type"""


class ChatStreamingMessageToolCallFunctionTypedDict(TypedDict):
    r"""Function call details"""

    name: NotRequired[str]
    r"""Function name"""
    arguments: NotRequired[str]
    r"""Function arguments as JSON string"""


class ChatStreamingMessageToolCallFunction(BaseModel):
    r"""Function call details"""

    name: Optional[str] = None
    r"""Function name"""

    arguments: Optional[str] = None
    r"""Function arguments as JSON string"""


class ChatStreamingMessageToolCallTypedDict(TypedDict):
    r"""Tool call delta for streaming responses"""

    index: float
    r"""Tool call index in the array"""
    id: NotRequired[str]
    r"""Tool call identifier"""
    type: NotRequired[ChatStreamingMessageToolCallType]
    r"""Tool call type"""
    function: NotRequired[ChatStreamingMessageToolCallFunctionTypedDict]
    r"""Function call details"""


class ChatStreamingMessageToolCall(BaseModel):
    r"""Tool call delta for streaming responses"""

    index: float
    r"""Tool call index in the array"""

    id: Optional[str] = None
    r"""Tool call identifier"""

    type: Optional[ChatStreamingMessageToolCallType] = None
    r"""Tool call type"""

    function: Optional[ChatStreamingMessageToolCallFunction] = None
    r"""Function call details"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .inputaudio import InputAudio, InputAudioTypedDict
from .inputfile import InputFile, InputFileTypedDict
from .inputtext import InputText, InputTextTypedDict
from .inputvideo import InputVideo, InputVideoTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import get_discriminator, validate_open_enum
from pydantic import Discriminator, Tag, model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


EasyInputMessageTypeMessage = Literal["message",]


EasyInputMessageRoleDeveloper = Literal["developer",]


EasyInputMessageRoleAssistant = Literal["assistant",]


EasyInputMessageRoleSystem = Literal["system",]


EasyInputMessageRoleUser = Literal["user",]


EasyInputMessageRoleUnionTypedDict = TypeAliasType(
    "EasyInputMessageRoleUnionTypedDict",
    Union[
        EasyInputMessageRoleUser,
        EasyInputMessageRoleSystem,
        EasyInputMessageRoleAssistant,
        EasyInputMessageRoleDeveloper,
    ],
)


EasyInputMessageRoleUnion = TypeAliasType(
    "EasyInputMessageRoleUnion",
    Union[
        EasyInputMessageRoleUser,
        EasyInputMessageRoleSystem,
        EasyInputMessageRoleAssistant,
        EasyInputMessageRoleDeveloper,
    ],
)


EasyInputMessageContentType = Literal["input_image",]


EasyInputMessageDetail = Union[
    Literal[
        "auto",
        "high",
        "low",
    ],
    UnrecognizedStr,
]


class EasyInputMessageContentInputImageTypedDict(TypedDict):
    r"""Image input content item"""

    type: EasyInputMessageContentType
    detail: EasyInputMessageDetail
    image_url: NotRequired[Nullable[str]]


class EasyInputMessageContentInputImage(BaseModel):
    r"""Image input content item"""

    type: EasyInputMessageContentType

    detail: Annotated[EasyInputMessageDetail, PlainValidator(validate_open_enum(False))]

    image_url: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["image_url"]
        nullable_fields = ["image_url"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


EasyInputMessageContentUnion1TypedDict = TypeAliasType(
    "EasyInputMessageContentUnion1TypedDict",
    Union[
        InputTextTypedDict,
        InputAudioTypedDict,
        InputVideoTypedDict,
        EasyInputMessageContentInputImageTypedDict,
        InputFileTypedDict,
    ],
)


EasyInputMessageContentUnion1 = Annotated[
    Union[
        Annotated[InputText, Tag("input_text")],
        Annotated[EasyInputMessageContentInputImage, Tag("input_image")],
        Annotated[InputFile, Tag("input_file")],
        Annotated[InputAudio, Tag("input_audio")],
        Annotated[InputVideo, Tag("input_video")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


EasyInputMessageContentUnion2TypedDict = TypeAliasType(
    "EasyInputMessageContentUnion2TypedDict",
    Union[List[EasyInputMessageContentUnion1TypedDict], str, Any],
)


EasyInputMessageContentUnion2 = TypeAliasType(
    "EasyInputMessageContentUnion2",
    Union[List[EasyInputMessageContentUnion1], str, Any],
)


EasyInputMessagePhaseFinalAnswer = Literal["final_answer",]


EasyInputMessagePhaseCommentary = Literal["commentary",]


EasyInputMessagePhaseUnionTypedDict = TypeAliasType(
    "EasyInputMessagePhaseUnionTypedDict",
    Union[EasyInputMessagePhaseCommentary, EasyInputMessagePhaseFinalAnswer, Any],
)
r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


EasyInputMessagePhaseUnion = TypeAliasType(
    "EasyInputMessagePhaseUnion",
    Union[EasyInputMessagePhaseCommentary, EasyInputMessagePhaseFinalAnswer, Any],
)
r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


class EasyInputMessageTypedDict(TypedDict):
    role: EasyInputMessageRoleUnionTypedDict
    type: NotRequired[EasyInputMessageTypeMessage]
    content: NotRequired[Nullable[EasyInputMessageContentUnion2TypedDict]]
    phase: NotRequired[Nullable[EasyInputMessagePhaseUnionTypedDict]]
    r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


class EasyInputMessage(BaseModel):
    role: EasyInputMessageRoleUnion

    type: Optional[EasyInputMessageTypeMessage] = None

    content: OptionalNullable[EasyInputMessageContentUnion2] = UNSET

    phase: OptionalNullable[EasyInputMessagePhaseUnion] = UNSET
    r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["type", "content", "phase"]
        nullable_fields = ["content", "phase"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


OutputDatetimeItemType = Literal["openrouter:datetime",]


OutputDatetimeItemStatus = Union[
    Literal[
        "completed",
        "in_progress",
        "incomplete",
    ],
    UnrecognizedStr,
]


class OutputDatetimeItemTypedDict(TypedDict):
    r"""An openrouter:datetime server tool output item"""

    type: OutputDatetimeItemType
    status: OutputDatetimeItemStatus
    datetime: str
    r"""ISO 8601 datetime string"""
    timezone: str
    r"""IANA timezone name"""
    id: NotRequired[str]


class OutputDatetimeItem(BaseModel):
    r"""An openrouter:datetime server tool output item"""

    type: OutputDatetimeItemType

    status: Annotated[
        OutputDatetimeItemStatus, PlainValidator(validate_open_enum(False))
    ]

    datetime: str
    r"""ISO 8601 datetime string"""

    timezone: str
    r"""IANA timezone name"""

    id: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from openrouter.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class BulkAssignMembersToGuardrailGlobalsTypedDict(TypedDict):
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class BulkAssignMembersToGuardrailGlobals(BaseModel):
    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class BulkAssignMembersToGuardrailRequestBodyTypedDict(TypedDict):
    member_user_ids: List[str]
    r"""Array of member user IDs to assign to the guardrail"""


class BulkAssignMembersToGuardrailRequestBody(BaseModel):
    member_user_ids: List[str]
    r"""Array of member user IDs to assign to the guardrail"""


class BulkAssignMembersToGuardrailRequestTypedDict(TypedDict):
    id: str
    r"""The unique identifier of the guardrail"""
    request_body: BulkAssignMembersToGuardrailRequestBodyTypedDict
    http_referer: NotRequired[str]
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """
    x_open_router_title: NotRequired[str]
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """
    x_open_router_categories: NotRequired[str]
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class BulkAssignMembersToGuardrailRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The unique identifier of the guardrail"""

    request_body: Annotated[
        BulkAssignMembersToGuardrailRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    http_referer: Annotated[
        Optional[str],
        pydantic.Field(alias="HTTP-Referer"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app identifier should be your app's URL and is used as the primary identifier for rankings.
    This is used to track API usage per application.

    """

    x_open_router_title: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Title"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The app display name allows you to customize how your app appears in OpenRouter's dashboard.

    """

    x_open_router_categories: Annotated[
        Optional[str],
        pydantic.Field(alias="X-OpenRouter-Categories"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

    """


class BulkAssignMembersToGuardrailResponseTypedDict(TypedDict):
    r"""Assignment result"""

    assigned_count: float
    r"""Number of members successfully assigned"""


class BulkAssignMembersToGuardrailResponse(BaseModel):
    r"""Assignment result"""

    assigned_count: float
    r"""Number of members successfully assigned"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chataudiooutput import ChatAudioOutput, ChatAudioOutputTypedDict
from .chatstreamtoolcall import ChatStreamToolCall, ChatStreamToolCallTypedDict
from .reasoningdetailunion import ReasoningDetailUnion, ReasoningDetailUnionTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


ChatStreamDeltaRole = Literal["assistant",]
r"""The role of the message author"""


class ChatStreamDeltaTypedDict(TypedDict):
    r"""Delta changes in streaming response"""

    role: NotRequired[ChatStreamDeltaRole]
    r"""The role of the message author"""
    content: NotRequired[Nullable[str]]
    r"""Message content delta"""
    reasoning: NotRequired[Nullable[str]]
    r"""Reasoning content delta"""
    refusal: NotRequired[Nullable[str]]
    r"""Refusal message delta"""
    tool_calls: NotRequired[List[ChatStreamToolCallTypedDict]]
    r"""Tool calls delta"""
    reasoning_details: NotRequired[List[ReasoningDetailUnionTypedDict]]
    r"""Reasoning details for extended thinking models"""
    audio: NotRequired[ChatAudioOutputTypedDict]


class ChatStreamDelta(BaseModel):
    r"""Delta changes in streaming response"""

    role: Optional[ChatStreamDeltaRole] = None
    r"""The role of the message author"""

    content: OptionalNullable[str] = UNSET
    r"""Message content delta"""

    reasoning: OptionalNullable[str] = UNSET
    r"""Reasoning content delta"""

    refusal: OptionalNullable[str] = UNSET
    r"""Refusal message delta"""

    tool_calls: Optional[List[ChatStreamToolCall]] = None
    r"""Tool calls delta"""

    reasoning_details: Optional[List[ReasoningDetailUnion]] = None
    r"""Reasoning details for extended thinking models"""

    audio: Optional[ChatAudioOutput] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "role",
            "content",
            "reasoning",
            "refusal",
            "tool_calls",
            "reasoning_details",
            "audio",
        ]
        nullable_fields = ["content", "reasoning", "refusal"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatcontentvideoinput import ChatContentVideoInput, ChatContentVideoInputTypedDict
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict, deprecated


LegacyChatContentVideoType = Literal["input_video",]


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class LegacyChatContentVideoTypedDict(TypedDict):
    r"""Video input content part (legacy format - deprecated)"""

    type: LegacyChatContentVideoType
    video_url: ChatContentVideoInputTypedDict
    r"""Video input object"""


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class LegacyChatContentVideo(BaseModel):
    r"""Video input content part (legacy format - deprecated)"""

    type: LegacyChatContentVideoType

    video_url: ChatContentVideoInput
    r"""Video input object"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .namedtoolchoice import NamedToolChoice, NamedToolChoiceTypedDict
from typing import Literal, Union
from typing_extensions import TypeAliasType


ToolChoiceOptionRequired = Literal["required",]


ToolChoiceOptionAuto = Literal["auto",]


ToolChoiceOptionNone = Literal["none",]


ToolChoiceOptionTypedDict = TypeAliasType(
    "ToolChoiceOptionTypedDict",
    Union[
        NamedToolChoiceTypedDict,
        ToolChoiceOptionNone,
        ToolChoiceOptionAuto,
        ToolChoiceOptionRequired,
    ],
)
r"""Tool choice configuration"""


ToolChoiceOption = TypeAliasType(
    "ToolChoiceOption",
    Union[
        NamedToolChoice,
        ToolChoiceOptionNone,
        ToolChoiceOptionAuto,
        ToolChoiceOptionRequired,
    ],
)
r"""Tool choice configuration"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openairesponsesannotation import (
    OpenAIResponsesAnnotation,
    OpenAIResponsesAnnotationTypedDict,
)
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


AnnotationAddedEventType = Literal["response.output_text.annotation.added",]


class AnnotationAddedEventTypedDict(TypedDict):
    r"""Event emitted when a text annotation is added to output"""

    type: AnnotationAddedEventType
    output_index: float
    item_id: str
    content_index: float
    sequence_number: float
    annotation_index: float
    annotation: OpenAIResponsesAnnotationTypedDict


class AnnotationAddedEvent(BaseModel):
    r"""Event emitted when a text annotation is added to output"""

    type: AnnotationAddedEventType

    output_index: float

    item_id: str

    content_index: float

    sequence_number: float

    annotation_index: float

    annotation: OpenAIResponsesAnnotation



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatmessagecontentitemcachecontrol import (
    ChatMessageContentItemCacheControl,
    ChatMessageContentItemCacheControlTypedDict,
)
from openrouter.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


ChatMessageContentItemTextType = Literal["text",]


class ChatMessageContentItemTextTypedDict(TypedDict):
    r"""Text content part"""

    type: ChatMessageContentItemTextType
    text: str
    cache_control: NotRequired[ChatMessageContentItemCacheControlTypedDict]
    r"""Cache control for the content part"""


class ChatMessageContentItemText(BaseModel):
    r"""Text content part"""

    type: ChatMessageContentItemTextType

    text: str

    cache_control: Optional[ChatMessageContentItemCacheControl] = None
    r"""Cache control for the content part"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal, Optional
from typing_extensions import NotRequired, TypedDict


WebSearchServerToolOpenRouterType = Literal["openrouter:web_search",]


class WebSearchServerToolOpenRouterParametersTypedDict(TypedDict):
    max_results: NotRequired[float]
    r"""Maximum number of search results to return per search call. Defaults to 5."""
    max_total_results: NotRequired[float]
    r"""Maximum total number of search results across all search calls in a single request. Once this limit is reached, the tool will stop returning new results."""


class WebSearchServerToolOpenRouterParameters(BaseModel):
    max_results: Optional[float] = None
    r"""Maximum number of search results to return per search call. Defaults to 5."""

    max_total_results: Optional[float] = None
    r"""Maximum total number of search results across all search calls in a single request. Once this limit is reached, the tool will stop returning new results."""


class WebSearchServerToolOpenRouterTypedDict(TypedDict):
    r"""OpenRouter built-in server tool: searches the web for current information"""

    type: WebSearchServerToolOpenRouterType
    parameters: NotRequired[WebSearchServerToolOpenRouterParametersTypedDict]


class WebSearchServerToolOpenRouter(BaseModel):
    r"""OpenRouter built-in server tool: searches the web for current information"""

    type: WebSearchServerToolOpenRouterType

    parameters: Optional[WebSearchServerToolOpenRouterParameters] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from openrouter import components, errors, operations, utils
from openrouter._hooks import HookContext
from openrouter.types import OptionalNullable, UNSET
from openrouter.utils import get_security_from_env
from openrouter.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional


class Analytics(BaseSDK):
    r"""Analytics and usage endpoints"""

    def get_user_activity(
        self,
        *,
        http_referer: Optional[str] = None,
        x_open_router_title: Optional[str] = None,
        x_open_router_categories: Optional[str] = None,
        date_: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetUserActivityResponse:
        r"""Get user activity grouped by endpoint

        Returns user activity data grouped by endpoint for the last 30 (completed) UTC days. [Management key](/docs/guides/overview/auth/management-api-keys) required.

        :param http_referer: The app identifier should be your app's URL and is used as the primary identifier for rankings.
            This is used to track API usage per application.

        :param x_open_router_title: The app display name allows you to customize how your app appears in OpenRouter's dashboard.

        :param x_open_router_categories: Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

        :param date_: Filter by a single UTC date in the last 30 days (YYYY-MM-DD format).
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetUserActivityRequest(
            http_referer=http_referer,
            x_open_router_title=x_open_router_title,
            x_open_router_categories=x_open_router_categories,
            date_=date_,
        )

        req = self._build_request(
            method="GET",
            path="/activity",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetUserActivityGlobals(
                http_referer=self.sdk_configuration.globals.http_referer,
                x_open_router_title=self.sdk_configuration.globals.x_open_router_title,
                x_open_router_categories=self.sdk_configuration.globals.x_open_router_categories,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getUserActivity",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.GetUserActivityResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                errors.ForbiddenResponseErrorData, http_res
            )
            raise errors.ForbiddenResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    async def get_user_activity_async(
        self,
        *,
        http_referer: Optional[str] = None,
        x_open_router_title: Optional[str] = None,
        x_open_router_categories: Optional[str] = None,
        date_: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetUserActivityResponse:
        r"""Get user activity grouped by endpoint

        Returns user activity data grouped by endpoint for the last 30 (completed) UTC days. [Management key](/docs/guides/overview/auth/management-api-keys) required.

        :param http_referer: The app identifier should be your app's URL and is used as the primary identifier for rankings.
            This is used to track API usage per application.

        :param x_open_router_title: The app display name allows you to customize how your app appears in OpenRouter's dashboard.

        :param x_open_router_categories: Comma-separated list of app categories (e.g. \"cli-agent,cloud-agent\"). Used for marketplace rankings.

        :param date_: Filter by a single UTC date in the last 30 days (YYYY-MM-DD format).
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetUserActivityRequest(
            http_referer=http_referer,
            x_open_router_title=x_open_router_title,
            x_open_router_categories=x_open_router_categories,
            date_=date_,
        )

        req = self._build_request_async(
            method="GET",
            path="/activity",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetUserActivityGlobals(
                http_referer=self.sdk_configuration.globals.http_referer,
                x_open_router_title=self.sdk_configuration.globals.x_open_router_title,
                x_open_router_categories=self.sdk_configuration.globals.x_open_router_categories,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getUserActivity",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.GetUserActivityResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                errors.ForbiddenResponseErrorData, http_res
            )
            raise errors.ForbiddenResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatcontentaudio import ChatContentAudio, ChatContentAudioTypedDict
from .chatcontentfile import ChatContentFile, ChatContentFileTypedDict
from .chatcontentimage import ChatContentImage, ChatContentImageTypedDict
from .chatcontenttext import ChatContentText, ChatContentTextTypedDict
from .chatcontentvideo import ChatContentVideo, ChatContentVideoTypedDict
from .legacy_chatcontentvideo import (
    LegacyChatContentVideo,
    LegacyChatContentVideoTypedDict,
)
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


ChatContentItems1TypedDict = TypeAliasType(
    "ChatContentItems1TypedDict",
    Union[LegacyChatContentVideoTypedDict, ChatContentVideoTypedDict],
)


ChatContentItems1 = Annotated[
    Union[
        Annotated[LegacyChatContentVideo, Tag("input_video")],
        Annotated[ChatContentVideo, Tag("video_url")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


ChatContentItemsTypedDict = TypeAliasType(
    "ChatContentItemsTypedDict",
    Union[
        ChatContentImageTypedDict,
        ChatContentAudioTypedDict,
        ChatContentFileTypedDict,
        ChatContentTextTypedDict,
        ChatContentItems1TypedDict,
    ],
)
r"""Content part for chat completion messages"""


ChatContentItems = TypeAliasType(
    "ChatContentItems",
    Union[
        ChatContentImage,
        ChatContentAudio,
        ChatContentFile,
        ChatContentText,
        ChatContentItems1,
    ],
)
r"""Content part for chat completion messages"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ChatMessageToolCallType = Literal["function",]


class ChatMessageToolCallFunctionTypedDict(TypedDict):
    name: str
    r"""Function name to call"""
    arguments: str
    r"""Function arguments as JSON string"""


class ChatMessageToolCallFunction(BaseModel):
    name: str
    r"""Function name to call"""

    arguments: str
    r"""Function arguments as JSON string"""


class ChatMessageToolCallTypedDict(TypedDict):
    r"""Tool call made by the assistant"""

    id: str
    r"""Tool call identifier"""
    type: ChatMessageToolCallType
    function: ChatMessageToolCallFunctionTypedDict


class ChatMessageToolCall(BaseModel):
    r"""Tool call made by the assistant"""

    id: str
    r"""Tool call identifier"""

    type: ChatMessageToolCallType

    function: ChatMessageToolCallFunction



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


NamedToolChoiceType = Literal["function",]


class NamedToolChoiceFunctionTypedDict(TypedDict):
    name: str
    r"""Function name to call"""


class NamedToolChoiceFunction(BaseModel):
    name: str
    r"""Function name to call"""


class NamedToolChoiceTypedDict(TypedDict):
    r"""Named tool choice for specific function"""

    type: NamedToolChoiceType
    function: NamedToolChoiceFunctionTypedDict


class NamedToolChoice(BaseModel):
    r"""Named tool choice for specific function"""

    type: NamedToolChoiceType

    function: NamedToolChoiceFunction



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


FunctionCallArgsDoneEventType = Literal["response.function_call_arguments.done",]


class FunctionCallArgsDoneEventTypedDict(TypedDict):
    r"""Event emitted when function call arguments streaming is complete"""

    type: FunctionCallArgsDoneEventType
    item_id: str
    output_index: float
    name: str
    arguments: str
    sequence_number: float


class FunctionCallArgsDoneEvent(BaseModel):
    r"""Event emitted when function call arguments streaming is complete"""

    type: FunctionCallArgsDoneEventType

    item_id: str

    output_index: float

    name: str

    arguments: str

    sequence_number: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatnamedtoolchoice import ChatNamedToolChoice, ChatNamedToolChoiceTypedDict
from typing import Literal, Union
from typing_extensions import TypeAliasType


ChatToolChoiceRequired = Literal["required",]


ChatToolChoiceAuto = Literal["auto",]


ChatToolChoiceNone = Literal["none",]


ChatToolChoiceTypedDict = TypeAliasType(
    "ChatToolChoiceTypedDict",
    Union[
        ChatNamedToolChoiceTypedDict,
        ChatToolChoiceNone,
        ChatToolChoiceAuto,
        ChatToolChoiceRequired,
    ],
)
r"""Tool choice configuration"""


ChatToolChoice = TypeAliasType(
    "ChatToolChoice",
    Union[
        ChatNamedToolChoice,
        ChatToolChoiceNone,
        ChatToolChoiceAuto,
        ChatToolChoiceRequired,
    ],
)
r"""Tool choice configuration"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import validate_open_enum
import pydantic
from pydantic.functional_validators import PlainValidator
from typing import Literal, Union
from typing_extensions import Annotated, TypedDict


InputAudioType = Literal["input_audio",]


InputAudioFormat = Union[
    Literal[
        "mp3",
        "wav",
    ],
    UnrecognizedStr,
]


class InputAudioInputAudioTypedDict(TypedDict):
    data: str
    format_: InputAudioFormat


class InputAudioInputAudio(BaseModel):
    data: str

    format_: Annotated[
        Annotated[InputAudioFormat, PlainValidator(validate_open_enum(False))],
        pydantic.Field(alias="format"),
    ]


class InputAudioTypedDict(TypedDict):
    r"""Audio input content item"""

    type: InputAudioType
    input_audio: InputAudioInputAudioTypedDict


class InputAudio(BaseModel):
    r"""Audio input content item"""

    type: InputAudioType

    input_audio: InputAudioInputAudio



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openairesponsesrefusalcontent import (
    OpenAIResponsesRefusalContent,
    OpenAIResponsesRefusalContentTypedDict,
)
from .openresponseseasyinputmessage import (
    OpenResponsesEasyInputMessage,
    OpenResponsesEasyInputMessageTypedDict,
)
from .openresponsesfunctioncalloutput import (
    OpenResponsesFunctionCallOutput,
    OpenResponsesFunctionCallOutputTypedDict,
)
from .openresponsesfunctiontoolcall import (
    OpenResponsesFunctionToolCall,
    OpenResponsesFunctionToolCallTypedDict,
)
from .openresponsesinputmessageitem import (
    OpenResponsesInputMessageItem,
    OpenResponsesInputMessageItemTypedDict,
)
from .openresponsesreasoning import (
    OpenResponsesReasoning,
    OpenResponsesReasoningTypedDict,
)
from .reasoningsummarytext import ReasoningSummaryText, ReasoningSummaryTextTypedDict
from .reasoningtextcontent import ReasoningTextContent, ReasoningTextContentTypedDict
from .responseoutputtext import ResponseOutputText, ResponseOutputTextTypedDict
from .responsesimagegenerationcall import (
    ResponsesImageGenerationCall,
    ResponsesImageGenerationCallTypedDict,
)
from .responsesoutputitemfilesearchcall import (
    ResponsesOutputItemFileSearchCall,
    ResponsesOutputItemFileSearchCallTypedDict,
)
from .responsesoutputitemfunctioncall import (
    ResponsesOutputItemFunctionCall,
    ResponsesOutputItemFunctionCallTypedDict,
)
from .responseswebsearchcalloutput import (
    ResponsesWebSearchCallOutput,
    ResponsesWebSearchCallOutputTypedDict,
)
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import get_discriminator, validate_open_enum
import pydantic
from pydantic import Discriminator, Tag, model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


OpenResponsesInputTypeReasoning = Literal["reasoning",]


OpenResponsesInputStatusInProgress2 = Literal["in_progress",]


OpenResponsesInputStatusIncomplete2 = Literal["incomplete",]


OpenResponsesInputStatusCompleted2 = Literal["completed",]


OpenResponsesInputStatusUnion2TypedDict = TypeAliasType(
    "OpenResponsesInputStatusUnion2TypedDict",
    Union[
        OpenResponsesInputStatusCompleted2,
        OpenResponsesInputStatusIncomplete2,
        OpenResponsesInputStatusInProgress2,
    ],
)


OpenResponsesInputStatusUnion2 = TypeAliasType(
    "OpenResponsesInputStatusUnion2",
    Union[
        OpenResponsesInputStatusCompleted2,
        OpenResponsesInputStatusIncomplete2,
        OpenResponsesInputStatusInProgress2,
    ],
)


OpenResponsesInputFormat = Union[
    Literal[
        "unknown",
        "openai-responses-v1",
        "azure-openai-responses-v1",
        "xai-responses-v1",
        "anthropic-claude-v1",
        "google-gemini-v1",
    ],
    UnrecognizedStr,
]
r"""The format of the reasoning content"""


class OpenResponsesInputReasoningTypedDict(TypedDict):
    r"""An output item containing reasoning"""

    type: OpenResponsesInputTypeReasoning
    id: str
    summary: Nullable[List[ReasoningSummaryTextTypedDict]]
    content: NotRequired[Nullable[List[ReasoningTextContentTypedDict]]]
    encrypted_content: NotRequired[Nullable[str]]
    status: NotRequired[OpenResponsesInputStatusUnion2TypedDict]
    signature: NotRequired[Nullable[str]]
    r"""A signature for the reasoning content, used for verification"""
    format_: NotRequired[Nullable[OpenResponsesInputFormat]]
    r"""The format of the reasoning content"""


class OpenResponsesInputReasoning(BaseModel):
    r"""An output item containing reasoning"""

    type: OpenResponsesInputTypeReasoning

    id: str

    summary: Nullable[List[ReasoningSummaryText]]

    content: OptionalNullable[List[ReasoningTextContent]] = UNSET

    encrypted_content: OptionalNullable[str] = UNSET

    status: Optional[OpenResponsesInputStatusUnion2] = None

    signature: OptionalNullable[str] = UNSET
    r"""A signature for the reasoning content, used for verification"""

    format_: Annotated[
        Annotated[
            OptionalNullable[OpenResponsesInputFormat],
            PlainValidator(validate_open_enum(False)),
        ],
        pydantic.Field(alias="format"),
    ] = UNSET
    r"""The format of the reasoning content"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "content",
            "encrypted_content",
            "status",
            "signature",
            "format",
        ]
        nullable_fields = [
            "content",
            "summary",
            "encrypted_content",
            "signature",
            "format",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


OpenResponsesInputRole = Literal["assistant",]


OpenResponsesInputTypeMessage = Literal["message",]


OpenResponsesInputStatusInProgress1 = Literal["in_progress",]


OpenResponsesInputStatusIncomplete1 = Literal["incomplete",]


OpenResponsesInputStatusCompleted1 = Literal["completed",]


OpenResponsesInputStatusUnion1TypedDict = TypeAliasType(
    "OpenResponsesInputStatusUnion1TypedDict",
    Union[
        OpenResponsesInputStatusCompleted1,
        OpenResponsesInputStatusIncomplete1,
        OpenResponsesInputStatusInProgress1,
    ],
)


OpenResponsesInputStatusUnion1 = TypeAliasType(
    "OpenResponsesInputStatusUnion1",
    Union[
        OpenResponsesInputStatusCompleted1,
        OpenResponsesInputStatusIncomplete1,
        OpenResponsesInputStatusInProgress1,
    ],
)


OpenResponsesInputContent1TypedDict = TypeAliasType(
    "OpenResponsesInputContent1TypedDict",
    Union[OpenAIResponsesRefusalContentTypedDict, ResponseOutputTextTypedDict],
)


OpenResponsesInputContent1 = Annotated[
    Union[
        Annotated[ResponseOutputText, Tag("output_text")],
        Annotated[OpenAIResponsesRefusalContent, Tag("refusal")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


OpenResponsesInputContent2TypedDict = TypeAliasType(
    "OpenResponsesInputContent2TypedDict",
    Union[List[OpenResponsesInputContent1TypedDict], str, Any],
)


OpenResponsesInputContent2 = TypeAliasType(
    "OpenResponsesInputContent2", Union[List[OpenResponsesInputContent1], str, Any]
)


OpenResponsesInputPhaseFinalAnswer = Literal["final_answer",]


OpenResponsesInputPhaseCommentary = Literal["commentary",]


OpenResponsesInputPhaseUnionTypedDict = TypeAliasType(
    "OpenResponsesInputPhaseUnionTypedDict",
    Union[OpenResponsesInputPhaseCommentary, OpenResponsesInputPhaseFinalAnswer, Any],
)
r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


OpenResponsesInputPhaseUnion = TypeAliasType(
    "OpenResponsesInputPhaseUnion",
    Union[OpenResponsesInputPhaseCommentary, OpenResponsesInputPhaseFinalAnswer, Any],
)
r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


class OpenResponsesInputMessageTypedDict(TypedDict):
    r"""An output message item"""

    id: str
    role: OpenResponsesInputRole
    type: OpenResponsesInputTypeMessage
    content: Nullable[OpenResponsesInputContent2TypedDict]
    status: NotRequired[OpenResponsesInputStatusUnion1TypedDict]
    phase: NotRequired[Nullable[OpenResponsesInputPhaseUnionTypedDict]]
    r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""


class OpenResponsesInputMessage(BaseModel):
    r"""An output message item"""

    id: str

    role: OpenResponsesInputRole

    type: OpenResponsesInputTypeMessage

    content: Nullable[OpenResponsesInputContent2]

    status: Optional[OpenResponsesInputStatusUnion1] = None

    phase: OptionalNullable[OpenResponsesInputPhaseUnion] = UNSET
    r"""The phase of an assistant message. Use `commentary` for an intermediate assistant message and `final_answer` for the final assistant message. For follow-up requests with models like `gpt-5.3-codex` and later, preserve and resend phase on all assistant messages. Omitting it can degrade performance. Not used for user messages."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["status", "phase"]
        nullable_fields = ["content", "phase"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


OpenResponsesInputUnion1TypedDict = TypeAliasType(
    "OpenResponsesInputUnion1TypedDict",
    Union[
        OpenResponsesEasyInputMessageTypedDict,
        OpenResponsesInputMessageItemTypedDict,
        ResponsesWebSearchCallOutputTypedDict,
        ResponsesOutputItemFileSearchCallTypedDict,
        ResponsesImageGenerationCallTypedDict,
        OpenResponsesFunctionCallOutputTypedDict,
        OpenResponsesFunctionToolCallTypedDict,
        OpenResponsesInputMessageTypedDict,
        ResponsesOutputItemFunctionCallTypedDict,
        OpenResponsesReasoningTypedDict,
        OpenResponsesInputReasoningTypedDict,
    ],
)


OpenResponsesInputUnion1 = TypeAliasType(
    "OpenResponsesInputUnion1",
    Union[
        OpenResponsesEasyInputMessage,
        OpenResponsesInputMessageItem,
        ResponsesWebSearchCallOutput,
        ResponsesOutputItemFileSearchCall,
        ResponsesImageGenerationCall,
        OpenResponsesFunctionCallOutput,
        OpenResponsesFunctionToolCall,
        OpenResponsesInputMessage,
        ResponsesOutputItemFunctionCall,
        OpenResponsesReasoning,
        OpenResponsesInputReasoning,
    ],
)


OpenResponsesInputUnionTypedDict = TypeAliasType(
    "OpenResponsesInputUnionTypedDict",
    Union[str, List[OpenResponsesInputUnion1TypedDict]],
)
r"""Input for a response request - can be a string or array of items"""


OpenResponsesInputUnion = TypeAliasType(
    "OpenResponsesInputUnion", Union[str, List[OpenResponsesInputUnion1]]
)
r"""Input for a response request - can be a string or array of items"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .websearchstatus import WebSearchStatus
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from openrouter.utils import get_discriminator, validate_open_enum
from pydantic import Discriminator, Tag, model_serializer
from pydantic.functional_validators import PlainValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


TypeWebSearchCall = Literal["web_search_call",]


TypeFindInPage = Literal["find_in_page",]


class ActionFindInPageTypedDict(TypedDict):
    type: TypeFindInPage
    pattern: str
    url: str


class ActionFindInPage(BaseModel):
    type: TypeFindInPage

    pattern: str

    url: str


TypeOpenPage = Literal["open_page",]


class ActionOpenPageTypedDict(TypedDict):
    type: TypeOpenPage
    url: NotRequired[Nullable[str]]


class ActionOpenPage(BaseModel):
    type: TypeOpenPage

    url: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["url"]
        nullable_fields = ["url"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


TypeSearch = Literal["search",]


TypeURL = Literal["url",]


class SourceTypedDict(TypedDict):
    type: TypeURL
    url: str


class Source(BaseModel):
    type: TypeURL

    url: str


class ActionSearchTypedDict(TypedDict):
    type: TypeSearch
    query: str
    queries: NotRequired[List[str]]
    sources: NotRequired[List[SourceTypedDict]]


class ActionSearch(BaseModel):
    type: TypeSearch

    query: str

    queries: Optional[List[str]] = None

    sources: Optional[List[Source]] = None


ActionTypedDict = TypeAliasType(
    "ActionTypedDict",
    Union[ActionOpenPageTypedDict, ActionFindInPageTypedDict, ActionSearchTypedDict],
)


Action = Annotated[
    Union[
        Annotated[ActionSearch, Tag("search")],
        Annotated[ActionOpenPage, Tag("open_page")],
        Annotated[ActionFindInPage, Tag("find_in_page")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class OutputWebSearchCallItemTypedDict(TypedDict):
    type: TypeWebSearchCall
    id: str
    action: ActionTypedDict
    status: WebSearchStatus


class OutputWebSearchCallItem(BaseModel):
    type: TypeWebSearchCall

    id: str

    action: Action

    status: Annotated[WebSearchStatus, PlainValidator(validate_open_enum(False))]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .assistantmessageimages import (
    AssistantMessageImages,
    AssistantMessageImagesTypedDict,
)
from .chatcompletionaudiooutput import (
    ChatCompletionAudioOutput,
    ChatCompletionAudioOutputTypedDict,
)
from .chatmessagecontentitem import (
    ChatMessageContentItem,
    ChatMessageContentItemTypedDict,
)
from .chatmessagetoolcall import ChatMessageToolCall, ChatMessageToolCallTypedDict
from .reasoningdetailunion import ReasoningDetailUnion, ReasoningDetailUnionTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Any, List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


AssistantMessageRole = Literal["assistant",]


AssistantMessageContentTypedDict = TypeAliasType(
    "AssistantMessageContentTypedDict",
    Union[str, List[ChatMessageContentItemTypedDict], Any],
)
r"""Assistant message content"""


AssistantMessageContent = TypeAliasType(
    "AssistantMessageContent", Union[str, List[ChatMessageContentItem], Any]
)
r"""Assistant message content"""


class AssistantMessageTypedDict(TypedDict):
    r"""Assistant message for requests and responses"""

    role: AssistantMessageRole
    content: NotRequired[Nullable[AssistantMessageContentTypedDict]]
    r"""Assistant message content"""
    name: NotRequired[str]
    r"""Optional name for the assistant"""
    tool_calls: NotRequired[List[ChatMessageToolCallTypedDict]]
    r"""Tool calls made by the assistant"""
    refusal: NotRequired[Nullable[str]]
    r"""Refusal message if content was refused"""
    reasoning: NotRequired[Nullable[str]]
    r"""Reasoning output"""
    reasoning_details: NotRequired[List[ReasoningDetailUnionTypedDict]]
    r"""Reasoning details for extended thinking models"""
    images: NotRequired[List[AssistantMessageImagesTypedDict]]
    r"""Generated images from image generation models"""
    audio: NotRequired[ChatCompletionAudioOutputTypedDict]
    r"""Audio output data or reference"""


class AssistantMessage(BaseModel):
    r"""Assistant message for requests and responses"""

    role: AssistantMessageRole

    content: OptionalNullable[AssistantMessageContent] = UNSET
    r"""Assistant message content"""

    name: Optional[str] = None
    r"""Optional name for the assistant"""

    tool_calls: Optional[List[ChatMessageToolCall]] = None
    r"""Tool calls made by the assistant"""

    refusal: OptionalNullable[str] = UNSET
    r"""Refusal message if content was refused"""

    reasoning: OptionalNullable[str] = UNSET
    r"""Reasoning output"""

    reasoning_details: Optional[List[ReasoningDetailUnion]] = None
    r"""Reasoning details for extended thinking models"""

    images: Optional[List[AssistantMessageImages]] = None
    r"""Generated images from image generation models"""

    audio: Optional[ChatCompletionAudioOutput] = None
    r"""Audio output data or reference"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "content",
            "name",
            "tool_calls",
            "refusal",
            "reasoning",
            "reasoning_details",
            "images",
            "audio",
        ]
        nullable_fields = ["content", "refusal", "reasoning"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ResponseFormatJSONObjectType = Literal["json_object",]


class ResponseFormatJSONObjectTypedDict(TypedDict):
    r"""JSON object response format"""

    type: ResponseFormatJSONObjectType


class ResponseFormatJSONObject(BaseModel):
    r"""JSON object response format"""

    type: ResponseFormatJSONObjectType



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing_extensions import TypedDict


class VideoInputTypedDict(TypedDict):
    r"""Video input object"""

    url: str
    r"""URL of the video (data: URLs supported)"""


class VideoInput(BaseModel):
    r"""Video input object"""

    url: str
    r"""URL of the video (data: URLs supported)"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .outputfilesearchcallitem import (
    OutputFileSearchCallItem,
    OutputFileSearchCallItemTypedDict,
)
from .outputfunctioncallitem import (
    OutputFunctionCallItem,
    OutputFunctionCallItemTypedDict,
)
from .outputimagegenerationcallitem import (
    OutputImageGenerationCallItem,
    OutputImageGenerationCallItemTypedDict,
)
from .outputmessageitem import OutputMessageItem, OutputMessageItemTypedDict
from .outputreasoningitem import OutputReasoningItem, OutputReasoningItemTypedDict
from .outputservertoolitem import OutputServerToolItem, OutputServerToolItemTypedDict
from .outputwebsearchcallitem import (
    OutputWebSearchCallItem,
    OutputWebSearchCallItemTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


OutputItemsTypedDict = TypeAliasType(
    "OutputItemsTypedDict",
    Union[
        OutputWebSearchCallItemTypedDict,
        OutputFileSearchCallItemTypedDict,
        OutputImageGenerationCallItemTypedDict,
        OutputServerToolItemTypedDict,
        OutputMessageItemTypedDict,
        OutputFunctionCallItemTypedDict,
        OutputReasoningItemTypedDict,
    ],
)
r"""An output item from the response"""


OutputItems = TypeAliasType(
    "OutputItems",
    Union[
        OutputWebSearchCallItem,
        OutputFileSearchCallItem,
        OutputImageGenerationCallItem,
        OutputServerToolItem,
        OutputMessageItem,
        OutputFunctionCallItem,
        OutputReasoningItem,
    ],
)
r"""An output item from the response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatcontentvideoinput import ChatContentVideoInput, ChatContentVideoInputTypedDict
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ChatContentVideoType = Literal["video_url",]


class ChatContentVideoTypedDict(TypedDict):
    r"""Video input content part"""

    type: ChatContentVideoType
    video_url: ChatContentVideoInputTypedDict
    r"""Video input object"""


class ChatContentVideo(BaseModel):
    r"""Video input content part"""

    type: ChatContentVideoType

    video_url: ChatContentVideoInput
    r"""Video input object"""
